import * as t from "io-ts";
import { Map } from "immutable";
interface KeyValuePair<K, V> {
    key: K;
    value: V;
}
declare function KeyValuePair<CKey extends t.Mixed, CValue extends t.Mixed>(keyCodec: CKey, valueCodec: CValue): t.TypeC<{
    key: CKey;
    value: CValue;
}>;
/**
 * A codec for the Map type of immutablejs.
 * @param keyCodec The codec for the key.
 * @param valueCodec The codec for the value.
 * @param format Configures the format of the encoded values.
 * Object: {
 *  key1: "value"
 * }
 * TupleArray: [
 *  ["key1", "value1"],
 *  ["key1", "value1"]
 * ]
 * KeyValueArray: [
 *  {key: "key1", value: "value1"},
 *  {key: "key2", value: "value2"},
 * ]
 */
export declare function map<CValue extends t.Mixed>(keyCodec: t.StringC, valueCodec: CValue, format: "object"): t.Type<Map<string, t.TypeOf<CValue>>, object>;
export declare function map<CKey extends t.Mixed, CValue extends t.Mixed>(keyCodec: CKey, valueCodec: CValue, format: "tuples"): t.Type<Map<string, CValue>, Array<[t.OutputOf<CKey>, t.OutputOf<CValue>]>>;
export declare function map<CKey extends t.Mixed, CValue extends t.Mixed>(keyCodec: CKey, valueCodec: CValue): t.Type<Map<t.TypeOf<CKey>, t.TypeOf<CValue>>, Array<[t.OutputOf<CKey>, t.OutputOf<CValue>]>>;
export declare function map<CKey extends t.Mixed, CValue extends t.Mixed>(keyCodec: CKey, valueCodec: CValue, format: "pairs"): t.Type<Map<t.TypeOf<CKey>, t.TypeOf<CValue>>, Array<KeyValuePair<t.OutputOf<CKey>, t.OutputOf<CValue>>>>;
export {};
//# sourceMappingURL=map.d.ts.map