"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var t = __importStar(require("io-ts"));
var immutable_1 = require("immutable");
var Either_1 = require("fp-ts/lib/Either");
function KeyValuePair(keyCodec, valueCodec) {
    return t.type({
        key: keyCodec,
        value: valueCodec
    }, "KeyValuePair");
}
// <editor-fold desc="validate">
function validateTuples(keyCodec, valueCodec, u, context) {
    var validatedArray = t.array(t.tuple([keyCodec, valueCodec])).validate(u, context);
    if (Either_1.isLeft(validatedArray)) {
        return t.failure(u, context);
    }
    var decoded = immutable_1.Map(validatedArray.right);
    return t.success(decoded);
}
function validatePairs(keyCodec, valueCodec, u, context) {
    var validatedArray = t.array(KeyValuePair(keyCodec, valueCodec)).validate(u, context);
    if (Either_1.isLeft(validatedArray)) {
        return t.failure(u, context);
    }
    var tupleArray = validatedArray.right.map(function (pair) { return [pair.key, pair.value]; });
    var decoded = immutable_1.Map(tupleArray);
    return t.success(decoded);
}
function validateObject(keyCodec, valueCodec, u, context) {
    var decodedRecord = t.record(keyCodec, valueCodec).decode(u);
    var map = immutable_1.Map().asMutable();
    if (Either_1.isLeft(decodedRecord)) {
        return t.failure(u, context);
    }
    for (var _i = 0, _a = Object.keys(decodedRecord.right); _i < _a.length; _i++) {
        var key = _a[_i];
        map.set(key, decodedRecord.right[key]);
    }
    return t.success(map.asImmutable());
}
// </editor-fold>
// <editor-fold desc="encode">
function encodeAsTuples(keyCodec, valueCodec, input) {
    return input.mapEntries(function (e) { return [keyCodec.encode(e[0]), valueCodec.encode(e[1])]; }).toArray();
}
function encodeAsPairs(keyCodec, valueCodec, input) {
    return input.toArray().map(function (e) { return ({
        key: keyCodec.encode(e[0]),
        value: valueCodec.encode(e[1])
    }); });
}
function encodeAsObject(keyCodec, valueCodec, input) {
    return input.mapEntries(function (e) { return [keyCodec.encode(e[0]), valueCodec.encode(e[1])]; }).toObject();
}
function map(keyCodec, valueCodec, format) {
    if (format === void 0) { format = "tuples"; }
    return new t.Type("Map<" + keyCodec.name + ", " + valueCodec.name + ">", function (u) {
        if (!immutable_1.Map.isMap(u)) {
            return false;
        }
        var allKeysAreCorrect = u.keySeq().every(keyCodec.is);
        var allValuesAreCorrect = u.valueSeq().every(valueCodec.is);
        return allKeysAreCorrect && allValuesAreCorrect;
    }, function (u, c) {
        switch (format) {
            case "tuples":
                return validateTuples(keyCodec, valueCodec, u, c);
            case "pairs":
                return validatePairs(keyCodec, valueCodec, u, c);
            case "object":
                return validateObject(keyCodec, valueCodec, u, c);
        }
    }, function (i) {
        switch (format) {
            case "tuples":
                return encodeAsTuples(keyCodec, valueCodec, i);
            case "pairs":
                return encodeAsPairs(keyCodec, valueCodec, i);
            default:
                return encodeAsObject(keyCodec, valueCodec, i);
        }
    });
}
exports.map = map;
//# sourceMappingURL=map.js.map